"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Refactorer = void 0;
const ts_morph_1 = require("ts-morph");
const complexity_service_1 = require("../services/complexity.service");
class Refactorer {
    constructor(projectService) {
        this.refactorProposals = [];
        this.projectService = projectService;
    }
    /**
     * Refactor nodes if it needed
     * apply transformers
     * then map refactored node in refactor proposal
     * @returns {void}
     */
    apply() {
        this.nodes = this.projectService.getNodesOfKinds(this.REFACTORED_NODE_KIND)
            .filter(n => this.refactorNeeded(n))
            .map((n, i) => {
            this.processOriginalNode(n);
            this.refactor(n);
            this.processRefactoredNode(n, i);
            return n;
        });
        if (this.transformers) {
            this.applyTransformers();
        }
    }
    /**
     * Apply additional tranformers to the node
     */
    applyTransformers() {
        this.nodes = this.transformers.map((t, i) => {
            const r = t.baseNode[t.nodeMethod]().transform(t.transformer);
            r.formatText();
            this.refactorProposals[i].newCode = r.getFullText();
            this.refactorProposals[i].usedTransformer = t;
            return r;
        });
    }
    /**
     * Compute and store information about the refactored node
     * @param n
     * @param i
     */
    processRefactoredNode(n, i) {
        this.refactorProposals[i].newComplexity = complexity_service_1.ComplexityService.getCpxFromSourceCode(n.getFullText());
        const existingRefactor = this.projectService.refactorProposals.find(er => er.id === this.refactorProposals[i].id);
        if (existingRefactor && existingRefactor.usedTransformer) {
            this.refactorProposals[i].oldCode = existingRefactor.oldCode;
            const transformer = existingRefactor.usedTransformer;
            n = n[transformer.nodeMethod]().transform(transformer.transformer);
        }
        n.formatText();
        this.refactorProposals[i].newCode = n.getFullText();
    }
    /**
     * Compute and store information about the original node
     * @param n
     */
    processOriginalNode(n) {
        n.formatText();
        this.refactorProposals.push({
            oldCode: n.getFullText(),
            newCode: undefined,
            title: n.getStructure()['name'],
            id: n.getStructure()['name'],
            oldComplexity: complexity_service_1.ComplexityService.getCpxFromSourceCode(n.getFullText()),
            newComplexity: undefined
        });
    }
    /**
     * Add a transformer
     * @param transformer
     * @returns {void}
     */
    addTransformer(transformer) {
        if (!this.transformers)
            this.transformers = [];
        if (!this.transformers.includes(transformer)) {
            this.transformers.push(transformer);
        }
    }
    /**
     * wrap a basic node into ts-morph model
     * use project to get source file and type checker
     * @param node
     * @param traversal the node to wrap
     * @returns {Node}
     */
    static wrapCurrentNode(node, traversal) {
        return ts_morph_1.createWrappedNode(traversal.visitChildren(), {
            sourceFile: node.getSourceFile().compilerNode,
            typeChecker: node.getProject().getTypeChecker().compilerObject,
        });
    }
}
exports.Refactorer = Refactorer;
