#!/usr/bin/env node
"use strict";
var _a, _b;
Object.defineProperty(exports, "__esModule", { value: true });
const worker_threads_1 = require("worker_threads");
const options_model_1 = require("./core/models/options.model");
const file_service_1 = require("./core/services/file.service");
const language_enum_1 = require("./core/enum/language.enum");
const ora = require('ora');
const path = require('path');
const spinner = ora();
const ARGS = process.argv.slice(2);
const PATH_TO_ANALYSE = (_a = ARGS[0]) !== null && _a !== void 0 ? _a : '.';
const LANGUAGE = (_b = ARGS[1]) !== null && _b !== void 0 ? _b : 'ts';
const ENABLE_MARKDOWN_REPORT = ARGS[2] === 'true';
const ENABLE_CONSOLE_REPORT = ARGS[3] === 'true';
const ENABLE_REFACTORING = ARGS[4] === 'true';
let pathToAnalyse;
if (path.isAbsolute(PATH_TO_ANALYSE)) {
    pathToAnalyse = PATH_TO_ANALYSE;
}
else {
    pathToAnalyse = `${process.cwd()}/${PATH_TO_ANALYSE}`.split('/').filter(e => e !== '.').join('/');
}
start()
    .then(exitCode => {
    process.exit(exitCode);
})
    .catch(err => {
    spinner.fail();
    console.log(err);
});
async function start() {
    options_model_1.Options.setOptions(process.cwd(), pathToAnalyse, __dirname);
    if (!ENABLE_CONSOLE_REPORT) {
        file_service_1.createOutDir();
    }
    spinner.start('AST generation');
    await useWorker(`${__dirname}/workers/ast-worker.js`, {
        pathCommand: process.cwd(),
        modifiedPath: pathToAnalyse,
        pathGeneseNodeJs: __dirname,
        language: LANGUAGE
    });
    spinner.succeed();
    spinner.start('Report generation');
    const reportResult = await useWorker(`${__dirname}/workers/report-worker.js`, {
        pathCommand: process.cwd(),
        modifiedPath: pathToAnalyse,
        pathGeneseNodeJs: __dirname,
        markdown: ENABLE_MARKDOWN_REPORT,
        consoleMode: ENABLE_CONSOLE_REPORT,
    });
    spinner.succeed();
    if (LANGUAGE === language_enum_1.Language.TS && !ENABLE_CONSOLE_REPORT && ENABLE_REFACTORING) {
        spinner.start('Refactoring generation');
        await useWorker(`${__dirname}/workers/refactoring-worker.js`, {
            pathCommand: options_model_1.Options.pathCommand,
            modifiedPath: pathToAnalyse,
            pathGeneseNodeJs: __dirname,
            astFolder: reportResult.astFolder
        });
        spinner.succeed();
    }
    file_service_1.deleteFile('./json-ast.json');
    if (reportResult.message && reportResult.message.length > 0) {
        console.log();
        if (typeof reportResult.message === 'object') {
            console.table(reportResult.message, ['filename', 'methodName', 'cpxIndex']);
        }
        else {
            console.log(reportResult.message);
        }
        if (ENABLE_CONSOLE_REPORT) {
            return 1;
        }
    }
    return 0;
}
function useWorker(filepath, data) {
    return new Promise((resolve, reject) => {
        const worker = new worker_threads_1.Worker(filepath, { workerData: data });
        worker.on('message', message => {
            resolve(message);
        });
        worker.on('error', reject);
        worker.on('exit', code => {
            if (code !== 0) {
                reject(new Error(`Worker stopped with exit code ${code}`));
            }
        });
    });
}
