"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AstFile = void 0;
const ast_file_service_1 = require("../../services/ast/ast-file.service");
const cpx_factors_model_1 = require("../../../core/models/cpx-factor/cpx-factors.model");
const ast_method_service_1 = require("../../services/ast/ast-method.service");
const chalk = require("chalk");
const nesting_cpx_model_1 = require("../../../core/models/cpx-factor/nesting-cpx.model");
const depth_cpx_model_1 = require("../../../core/models/cpx-factor/depth-cpx.model");
const tools_service_1 = require("../../../core/services/tools.service");
class AstFile {
    constructor() {
        this._astFolder = undefined; // The AstFolder which includes this AstFile
        this._astMethods = []; // The AstMethods included in this AstFile
        this._astNode = undefined; // The AstNode corresponding to the file itself
        this._astNodes = undefined; // Array of all the AstNodes which are children of this.AstNode (including itself)
        this._code = undefined; // The Code object corresponding to the AstFile
        this._complexitiesByStatus = undefined; // The file complexities spread by complexity status
        this._cpxFactors = undefined; // The complexity factors of the AstFile
        this._cyclomaticCpx = 0; // The complexity factors of the AstFile
        this._end = undefined; // The pos of the end of the source code
        this._name = undefined; // The name of the AstFile
        this._stats = undefined; // The statistics of the AstFile
    }
    // ---------------------------------------------------------------------------------
    //                                Getters and setters
    // ---------------------------------------------------------------------------------
    get astFolder() {
        return this._astFolder;
    }
    set astFolder(astFolder) {
        this._astFolder = astFolder;
    }
    get astMethods() {
        if (this._astMethods) {
            return this._astMethods;
        }
        return [];
    }
    set astMethods(astMethods) {
        this._astMethods = astMethods;
    }
    get astNode() {
        return this._astNode;
    }
    set astNode(astNode) {
        this._astNode = astNode;
    }
    get astNodes() {
        return this._astNodes;
    }
    set astNodes(astNodes) {
        this._astNodes = astNodes;
    }
    get code() {
        return this._code;
    }
    set code(code) {
        this._code = code;
    }
    get complexitiesByStatus() {
        return this._complexitiesByStatus;
    }
    set complexitiesByStatus(cpxByStatus) {
        this._complexitiesByStatus = cpxByStatus;
    }
    get cpxFactors() {
        return this._cpxFactors;
    }
    set cpxFactors(cpxFactors) {
        this._cpxFactors = cpxFactors;
    }
    get cyclomaticCpx() {
        return this._cyclomaticCpx;
    }
    set cyclomaticCpx(cyclomaticCpx) {
        this._cyclomaticCpx = cyclomaticCpx;
    }
    get end() {
        var _a, _b;
        return (_a = this._end) !== null && _a !== void 0 ? _a : (_b = this._astNode) === null || _b === void 0 ? void 0 : _b.end;
    }
    set end(end) {
        this._end = end;
    }
    get name() {
        return this._name;
    }
    set name(name) {
        this._name = name;
    }
    get stats() {
        if (!this._stats) {
            this._stats = new ast_file_service_1.AstFileService().getStats(this);
        }
        return this._stats;
    }
    set stats(stats) {
        this._stats = stats;
    }
    get text() {
        return this._code.text;
    }
    // ---------------------------------------------------------------------------------
    //                                  Other methods
    // ---------------------------------------------------------------------------------
    /**
     * Evaluates the complexities of the AstNodes and the AstMethods of this AstFile
     */
    evaluate() {
        this.cpxFactors = new cpx_factors_model_1.CpxFactors();
        const astMethodService = new ast_method_service_1.AstMethodService();
        this.astNode.evaluate();
        for (const method of this.astMethods) {
            method.evaluate();
            this.cpxFactors = this.cpxFactors.add(method.cpxFactors);
            this.cyclomaticCpx = this.cyclomaticCpx + method.cyclomaticCpx;
            this.complexitiesByStatus = astMethodService.addMethodCpxByStatus(this.complexitiesByStatus, method);
        }
    }
    /**
     * Evaluates the complexities of the AstNodes of this AstFile
     * But not based on methods
     */
    evaluateStandalone() {
        var _a, _b, _c, _d, _e;
        this.astMethods = [];
        this.cpxFactors = new cpx_factors_model_1.CpxFactors();
        this.astNode.evaluate();
        for (const node of this.astNodes) {
            node.cpxFactors.nesting = new nesting_cpx_model_1.NestingCpx();
            node.cpxFactors.depth = new depth_cpx_model_1.DepthCpx();
            for (const astNode of node.children) {
                if (astNode.intrinsicNestingCpx > 0) {
                    node.cpxFactors.depth = tools_service_1.addObjects(this.cpxFactors.depth, (_a = astNode.cpxFactors) === null || _a === void 0 ? void 0 : _a.depth);
                    node.cpxFactors.nesting = tools_service_1.addObjects(this.cpxFactors.nesting, (_c = (_b = astNode.parent) === null || _b === void 0 ? void 0 : _b.cpxFactors) === null || _c === void 0 ? void 0 : _c.nesting);
                }
                if (astNode.intrinsicDepthCpx > 0) {
                    node.cpxFactors.depth = tools_service_1.addObjects(this.cpxFactors.depth, (_e = (_d = astNode.parent) === null || _d === void 0 ? void 0 : _d.cpxFactors) === null || _e === void 0 ? void 0 : _e.depth);
                }
            }
            this.cpxFactors = this.cpxFactors.add(node.cpxFactors);
        }
    }
    /**
     * Logs the main elements of an AstFile
     * @param message       // Optional message
     */
    logg(message) {
        var _a, _b;
        console.log('-----------------------------');
        console.log(chalk.yellowBright(message !== null && message !== void 0 ? message : 'AST_FILE'));
        console.log(this.name);
        console.log('-----------------------------');
        console.log(chalk.blueBright('end :'), this.end);
        console.log(chalk.blueBright('text :'), this.text);
        console.log(chalk.blueBright('astNode :'), (_a = this.astNode) === null || _a === void 0 ? void 0 : _a.kind);
        console.log(chalk.blueBright('astFolder :'), (_b = this.astFolder) === null || _b === void 0 ? void 0 : _b.path);
    }
}
exports.AstFile = AstFile;
