"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AstFolder = void 0;
const complexities_by_status_interface_1 = require("../../interfaces/complexities-by-status.interface");
const cpx_factors_model_1 = require("../../../core/models/cpx-factor/cpx-factors.model");
const ast_folder_service_1 = require("../../services/ast/ast-folder.service");
const chalk = require("chalk");
class AstFolder {
    constructor() {
        this._astFiles = []; // The array of files of this folder (not in the subfolders)
        this._astFolderService = new ast_folder_service_1.AstFolderService(); // The service managing AstFolders
        this._children = []; // The subfolders of this folder
        this._complexitiesByStatus = new complexities_by_status_interface_1.ComplexitiesByStatus(); // The folder complexities spread by complexity status
        this._cpxFactors = undefined; // The complexity factors of the AstFolder
        this._cyclomaticCpx = 0; // The cyclomatic complexity of the AstFolder
        this._numberOfFiles = undefined; // The number of files of the AstFolder
        this._numberOfMethods = undefined; // The number of methods of the AstFolder
        this._parent = undefined; // The AstFolder corresponding to the parent folder of this AstFolder
        this._path = undefined; // The absolute path of this folder
        this._relativePath = undefined; // The relative path of this folder compared to the root folder of the analyse
        this._stats = undefined; // The stats corresponding to this folder
    }
    // ---------------------------------------------------------------------------------
    //                                Getters and setters
    // ---------------------------------------------------------------------------------
    get astFiles() {
        return this._astFiles;
    }
    set astFiles(astFiles) {
        this._astFiles = astFiles;
    }
    get children() {
        return this._children;
    }
    set children(children) {
        this._children = children;
    }
    get complexitiesByStatus() {
        return this._complexitiesByStatus;
    }
    set complexitiesByStatus(complexitiesByStatus) {
        this._complexitiesByStatus = complexitiesByStatus;
    }
    get cpxFactors() {
        if (this._cpxFactors) {
            return this._cpxFactors;
        }
        this.evaluate();
        return this._cpxFactors;
    }
    set cpxFactors(cpxFactors) {
        this._cpxFactors = cpxFactors;
    }
    get cyclomaticCpx() {
        return this._cyclomaticCpx;
    }
    set cyclomaticCpx(cyclomaticCpx) {
        this._cyclomaticCpx = cyclomaticCpx;
    }
    get numberOfFiles() {
        var _a;
        return (_a = this._numberOfFiles) !== null && _a !== void 0 ? _a : this._astFolderService.getNumberOfFiles(this);
    }
    set numberOfFiles(numberOfFiles) {
        this._numberOfFiles = numberOfFiles;
    }
    get numberOfMethods() {
        var _a;
        return (_a = this._numberOfMethods) !== null && _a !== void 0 ? _a : this._astFolderService.getNumberOfMethods(this);
    }
    set numberOfMethods(numberOfMethods) {
        this._numberOfMethods = numberOfMethods;
    }
    get parent() {
        return this._parent;
    }
    set parent(parent) {
        this._parent = parent;
    }
    get path() {
        return this._path;
    }
    set path(path) {
        this._path = path;
    }
    get relativePath() {
        var _a;
        return (_a = this._relativePath) !== null && _a !== void 0 ? _a : this._astFolderService.getRelativePath(this);
    }
    get stats() {
        return this._stats;
    }
    set stats(stats) {
        this._stats = stats;
    }
    // ---------------------------------------------------------------------------------
    //                                  Other methods
    // ---------------------------------------------------------------------------------
    /**
     * Evaluates and sets the complexities of the AstFiles of this AstFolder (including its subfolders)
     */
    evaluate() {
        this.cpxFactors = new cpx_factors_model_1.CpxFactors();
        this.evaluateCpxFactors(this);
        this.numberOfMethods = this._astFolderService.getNumberOfMethods(this);
        this.stats = this._astFolderService.calculateStats(this);
    }
    /**
     * Evaluates and sets the complexities of the AstFiles of this AstFolder
     * But not based on methods
     */
    evaluateStandalone() {
        this.cpxFactors = new cpx_factors_model_1.CpxFactors();
        const astFile = this.astFiles[0];
        astFile.evaluateStandalone();
        this.addCpx(astFile);
        this.numberOfMethods = 0;
        this.stats = this._astFolderService.calculateStats(this);
    }
    /**
     * Evaluates and sets the complexities of the AstFiles of a given AstFolder (including its subfolders)
     * @param astFolder     // The "parent" AstFolder
     */
    evaluateCpxFactors(astFolder) {
        for (const astFile of astFolder.astFiles) {
            astFile.evaluate();
            this.addCpx(astFile);
        }
        for (const childAstFolder of astFolder.children) {
            childAstFolder.evaluate();
            this.addCpx(childAstFolder);
        }
    }
    addCpx(element) {
        this.cpxFactors = this.cpxFactors.add(element.cpxFactors);
        this.cyclomaticCpx = this.cyclomaticCpx + element.cyclomaticCpx;
        this.complexitiesByStatus = this.complexitiesByStatus.add(element.complexitiesByStatus);
    }
    /**
     * Logs the main elements of the AstFolder
     * @param message       // An optional message
     */
    logg(message) {
        var _a, _b;
        console.log('-----------------------------');
        console.log(chalk.yellowBright(message !== null && message !== void 0 ? message : 'AST_FOLDER'));
        console.log(this.path);
        console.log('-----------------------------');
        console.log(chalk.blueBright('parent :'), (_a = this.parent) === null || _a === void 0 ? void 0 : _a.path);
        for (const astFile of this.astFiles) {
            const name = (_b = astFile === null || astFile === void 0 ? void 0 : astFile.name) !== null && _b !== void 0 ? _b : '';
            console.log(chalk.yellowBright(`  ${name}`));
            this.loggChildren(astFile === null || astFile === void 0 ? void 0 : astFile.astNode, `  `);
        }
    }
    /**
     * Logs the main elements of the children of the AstFolder's AstNode
     * @param astNode       // The AstNode of the AstFolder
     * @param indent        // The indentation of the current AstNode (in the log)
     */
    loggChildren(astNode, indent = '') {
        var _a;
        for (const childAstNode of astNode === null || astNode === void 0 ? void 0 : astNode.children) {
            const name = (_a = childAstNode === null || childAstNode === void 0 ? void 0 : childAstNode.name) !== null && _a !== void 0 ? _a : '';
            console.log(chalk.blueBright(`${indent}${childAstNode.kind}`), name);
            this.loggChildren(childAstNode, `${indent}  `);
        }
    }
}
exports.AstFolder = AstFolder;
