"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AstNode = void 0;
const syntax_kind_enum_1 = require("../../../core/enum/syntax-kind.enum");
const ast_service_1 = require("../../services/ast/ast.service");
const factor_category_service_1 = require("../../services/factor-category.service");
const cpx_factors_model_1 = require("../../../core/models/cpx-factor/cpx-factors.model");
const node_feature_enum_1 = require("../../enums/node-feature.enum");
const cpx_factors_1 = require("../../../core/const/cpx-factors");
const tools_service_1 = require("../../../core/services/tools.service");
const ast_node_service_1 = require("../../services/ast/ast-node.service");
const chalk = require("chalk");
const code_service_1 = require("../../services/code.service");
class AstNode {
    constructor() {
        this._astFile = undefined; // The AstFile containing the AST node of the AstNode
        this._astMethod = undefined; // The method at the root of the current ast (if this ast is inside a method)
        this._astNodeService = new ast_node_service_1.AstNodeService(); // The service managing AstNodes
        this._children = []; // The children AstNodes of the AstNode
        this._context = undefined; // The context of the AstNode
        this._cpxFactors = undefined; // The complexity factors of the AstNode
        this._cpxFactorsFromJsonAST = undefined; // The complexity factors added manually in JsonAST (have priority on calculated cpxFactors)
        this._cyclomaticCpx = 0; // The cyclomatic complexity of the AstNode
        this._end = 0; // The pos of the end of the source code of the AstNode in the source code of the AstFile
        this._factorCategory = undefined; // The NodeFeature of the node of the AstNode
        this._intrinsicDepthCpx = undefined; // The depth of the AstNode inside its method (not including its parent's depth)
        this._intrinsicNestingCpx = undefined; // The nesting of the AstNode inside its method (not including its parent's nesting)
        this._isCallback = undefined; // True if the astNode is a method with a Callback, false if not
        this._isRecursiveMethod = undefined; // True if the astNode is a recursive method, false if not
        this._kind = undefined; // The kind of the node ('MethodDeclaration, IfStatement, ...)
        this._lineEnd = undefined; // The issue of the line containing the character at the AstNode.end
        this._linePos = undefined; // The issue of the line containing the character at the AstNode.pos
        this._lineStart = undefined; // The issue of the line containing the character at the AstNode.start
        this._name = undefined; // The name of the AstNode
        this._pos = 0; // The pos of the beginning of the AST node, including spaces and comments before it. (start <= start)
        this._start = 0; // The pos of the beginning of the AST node, without spaces and comments before it. (start >= start)
        this._text = undefined; // The code of the AstNode
        this._type = undefined; // The type of the AstNode (if given)
    }
    // ---------------------------------------------------------------------------------
    //                                Getters and setters
    // ---------------------------------------------------------------------------------
    get aggregationCpx() {
        var _a;
        return (_a = this.cpxFactors) === null || _a === void 0 ? void 0 : _a.totalAggregation;
    }
    get astFile() {
        return this._astFile;
    }
    set astFile(astFile) {
        this._astFile = astFile;
    }
    get astMethod() {
        return this._astMethod;
    }
    set astMethod(astMethod) {
        this._astMethod = astMethod;
    }
    get atomicCpx() {
        var _a;
        return (_a = this.cpxFactors) === null || _a === void 0 ? void 0 : _a.totalAtomic;
    }
    get children() {
        return this._children;
    }
    set children(children) {
        this._children = children;
    }
    get context() {
        var _a;
        return (_a = this._context) !== null && _a !== void 0 ? _a : this._astNodeService.getContext(this);
    }
    set context(treeNode) {
        this._context = treeNode;
    }
    get cpxFactors() {
        return this._cpxFactors;
    }
    set cpxFactors(cpxFactors) {
        this._cpxFactors = cpxFactors;
    }
    get cpxFactorsFromJsonAST() {
        return this._cpxFactorsFromJsonAST;
    }
    set cpxFactorsFromJsonAST(cpxFactorsFromJsonAST) {
        this._cpxFactorsFromJsonAST = cpxFactorsFromJsonAST;
    }
    get cyclomaticCpx() {
        return this._cyclomaticCpx;
    }
    set cyclomaticCpx(cyclomaticCpx) {
        this._cyclomaticCpx = cyclomaticCpx;
    }
    get depthCpx() {
        var _a;
        return (_a = this.cpxFactors) === null || _a === void 0 ? void 0 : _a.totalDepth;
    }
    get end() {
        return this._end;
    }
    set end(end) {
        this._end = end;
    }
    get factorCategory() {
        var _a;
        return (_a = this._factorCategory) !== null && _a !== void 0 ? _a : new factor_category_service_1.FactorCategoryService().getNodeFeature(this.kind);
    }
    get firstSon() {
        return this.getSon(0);
    }
    get intrinsicDepthCpx() {
        return this._intrinsicDepthCpx;
    }
    set intrinsicDepthCpx(cpx) {
        this._intrinsicDepthCpx = cpx;
    }
    get intrinsicNestingCpx() {
        return this._intrinsicNestingCpx;
    }
    set intrinsicNestingCpx(cpx) {
        this._intrinsicNestingCpx = cpx;
    }
    get isCallback() {
        if (this._isCallback) {
            return this._isCallback;
        }
        this._isCallback = this._astNodeService.isCallback(this);
        return this._isCallback;
    }
    get isCallIdentifier() {
        return ast_service_1.Ast.isCallIdentifier(this) && this === this.parent.firstSon;
    }
    get isFunctionOrMethodDeclaration() {
        return this.factorCategory === node_feature_enum_1.NodeFeature.DECLARATION;
    }
    get isParam() {
        return ast_service_1.Ast.isParam(this);
    }
    get isRecursiveMethod() {
        if (this._isRecursiveMethod) {
            return this._isRecursiveMethod;
        }
        this._isRecursiveMethod = this._astNodeService.isRecursiveMethod(this);
        return this._isRecursiveMethod;
    }
    get kind() {
        return this._kind;
    }
    set kind(kind) {
        this._kind = kind;
    }
    get lineEnd() {
        var _a;
        if (this._lineEnd) {
            return this._lineEnd;
        }
        this._lineEnd = code_service_1.CodeService.getLineIssue((_a = this.astFile) === null || _a === void 0 ? void 0 : _a.code, this.end);
        return this._lineEnd;
    }
    get linePos() {
        var _a;
        if (this._linePos) {
            return this._linePos;
        }
        this._linePos = code_service_1.CodeService.getLineIssue((_a = this.astFile) === null || _a === void 0 ? void 0 : _a.code, this.pos);
        return this._linePos;
    }
    get lineStart() {
        var _a;
        if (this._lineStart) {
            return this._lineStart;
        }
        this._lineStart = code_service_1.CodeService.getLineIssue((_a = this.astFile) === null || _a === void 0 ? void 0 : _a.code, this.start);
        return this._lineStart;
    }
    get mayDefineContext() {
        return ast_service_1.Ast.mayDefineContext(this);
    }
    get name() {
        var _a;
        return (_a = this._name) !== null && _a !== void 0 ? _a : '';
    }
    set name(name) {
        this._name = name;
    }
    get nestingCpx() {
        var _a;
        return (_a = this.cpxFactors) === null || _a === void 0 ? void 0 : _a.totalNesting;
    }
    get parent() {
        return this._parent;
    }
    set parent(treeNode) {
        this._parent = treeNode;
    }
    get pos() {
        return this._pos;
    }
    set pos(pos) {
        this._pos = pos;
    }
    get start() {
        return this._start;
    }
    set start(start) {
        this._start = start;
    }
    get recursionCpx() {
        var _a;
        return (_a = this.cpxFactors) === null || _a === void 0 ? void 0 : _a.totalRecursion;
    }
    get secondSon() {
        return this.getSon(1);
    }
    get structuralCpx() {
        var _a;
        return (_a = this.cpxFactors) === null || _a === void 0 ? void 0 : _a.totalStructural;
    }
    get text() {
        var _a;
        return (_a = this._text) !== null && _a !== void 0 ? _a : this._astNodeService.getCode(this);
    }
    set text(text) {
        this._text = text;
    }
    get type() {
        return this._type;
    }
    set type(type) {
        this._type = type;
    }
    // ---------------------------------------------------------------------------------
    //                                  Other methods
    // ---------------------------------------------------------------------------------
    /**
     * Evaluates the complexity factors of this AstNode and its children
     */
    evaluate() {
        this.calculateAndSetCpxFactors();
        this.addParentCpx();
        for (const child of this._children) {
            child.evaluate();
        }
    }
    /**
     * Gets the xth son of this AstNode
     * @param sonNumber     // The number of the son (0 for the first one)
     */
    getSon(sonNumber) {
        return this.children[sonNumber];
    }
    /**
     * Calculates the complexity factors of the AstNode
     */
    calculateAndSetCpxFactors() {
        this.cpxFactors = new cpx_factors_model_1.CpxFactors();
        this.setGeneralCaseCpxFactors();
        this.setFunctionStructuralCpx();
        this.setRecursionOrCallbackCpxFactors();
        this.setElseCpxFactors();
        this.setRegexCpxFactors();
        this.setDepthCpxFactors();
        this.setAggregationCpxFactors();
        this.intrinsicNestingCpx = this.cpxFactors.totalNesting;
        this.intrinsicDepthCpx = this.cpxFactors.totalDepth;
        this.forceCpxFactors();
        return this._cpxFactors;
    }
    /**
     * Sets the nesting and structural complexities for "usual" cases
     */
    setGeneralCaseCpxFactors() {
        var _a;
        this.cpxFactors.nesting[this.factorCategory] = cpx_factors_1.cpxFactors.nesting[this.factorCategory];
        this.cpxFactors.structural[this.factorCategory] = cpx_factors_1.cpxFactors.structural[this.factorCategory];
        this.cpxFactors.atomic.node = (_a = cpx_factors_1.cpxFactors.atomic[this.factorCategory]) !== null && _a !== void 0 ? _a : cpx_factors_1.cpxFactors.atomic.node;
    }
    setFunctionStructuralCpx() {
        var _a;
        if (this.type === 'function' && ((_a = this.parent) === null || _a === void 0 ? void 0 : _a.kind) !== syntax_kind_enum_1.SyntaxKind.MethodDeclaration) {
            this.cpxFactors.structural.method = cpx_factors_1.cpxFactors.structural.method;
        }
    }
    forceCpxFactors() {
        if (this.cpxFactorsFromJsonAST) {
            for (const category of Object.keys(this.cpxFactorsFromJsonAST)) {
                for (const factor of Object.keys(this.cpxFactorsFromJsonAST[category])) {
                    this.cpxFactors[category][factor] = this.cpxFactorsFromJsonAST[category][factor];
                }
            }
        }
    }
    /**
     * Sets depth complexity factor
     * Example : array in array, like a[b[c]]
     */
    setDepthCpxFactors() {
        if (ast_service_1.Ast.isArrayIndex(this)) {
            this.cpxFactors.depth.arr = cpx_factors_1.cpxFactors.depth.arr;
        }
    }
    /**
     * Sets aggregation complexity factor
     */
    setAggregationCpxFactors() {
        if (ast_service_1.Ast.isArrayOfArray(this)) {
            this.cpxFactors.aggregation.arr = cpx_factors_1.cpxFactors.aggregation.arr;
        }
        else if (ast_service_1.Ast.isDifferentLogicDoor(this)) {
            this.cpxFactors.aggregation.differentLogicDoor = cpx_factors_1.cpxFactors.aggregation.differentLogicDoor;
        }
    }
    /**
     * Sets complexity factor for "else" case
     */
    setElseCpxFactors() {
        if (ast_service_1.Ast.isElseStatement(this)) {
            this.cpxFactors.structural.conditional = cpx_factors_1.cpxFactors.structural.conditional;
        }
        if (ast_service_1.Ast.isElseIfStatement(this)) {
            this.cpxFactors.nesting.conditional = 0;
        }
    }
    /**
     * Sets complexity factor for callbacks and recursions
     */
    setRecursionOrCallbackCpxFactors() {
        this.cpxFactors.recursion.recursivity = this.isRecursiveMethod ? cpx_factors_1.cpxFactors.recursion.recursivity : 0;
        this.cpxFactors.recursion.callback = this.isCallback ? cpx_factors_1.cpxFactors.recursion.callback : 0;
    }
    /**
     * Sets complexity factor for regex
     */
    setRegexCpxFactors() {
        if (this.factorCategory === node_feature_enum_1.NodeFeature.REGEX) {
            this.cpxFactors.aggregation.regex = +((this['text'].length - 2) * cpx_factors_1.cpxFactors.aggregation.regex).toFixed(2);
        }
    }
    /**
     * Sets the global nesting cpx of the node (the cpx from the node itself and from its parents)
     */
    addParentCpx() {
        var _a, _b, _c, _d, _e;
        if (this && this.parent && ((_b = (_a = this.parent) === null || _a === void 0 ? void 0 : _a.cpxFactors) === null || _b === void 0 ? void 0 : _b.nesting)) {
            this.cpxFactors.nesting = tools_service_1.addObjects(this.parent.cpxFactors.nesting, this.cpxFactors.nesting);
        }
        if (this && ((_c = this.parent) === null || _c === void 0 ? void 0 : _c.parent) && ((_e = (_d = this.parent) === null || _d === void 0 ? void 0 : _d.cpxFactors) === null || _e === void 0 ? void 0 : _e.depth)) {
            this.cpxFactors.depth = tools_service_1.addObjects(this.parent.cpxFactors.depth, this.cpxFactors.depth);
        }
    }
    /**
     * Logs the main information about the AstNode
     * @param message
     */
    logg(message) {
        console.log('-----------------------------');
        console.log(chalk.yellowBright(message !== null && message !== void 0 ? message : 'AST NODE'));
        console.log(this.kind, this.name);
        console.log('-----------------------------');
        console.log('pos', this.pos, 'end', this.end);
        console.log('text', this.text);
    }
}
exports.AstNode = AstNode;
