"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AstFolderMarkdownReportService = void 0;
const fs = require("fs-extra");
const eol = require("eol");
const Handlebars = require("handlebars");
const file_service_1 = require("../../../core/services/file.service");
const ast_folder_service_1 = require("../ast/ast-folder.service");
const options_model_1 = require("../../../core/models/options.model");
const ast_method_service_1 = require("../ast/ast-method.service");
/**
 * Service generating folders reports
 */
class AstFolderMarkdownReportService {
    constructor(astFolder) {
        this.astFolder = undefined; // The AstFolder relative to this service
        this.astFolderService = new ast_folder_service_1.AstFolderService(); // The service relative to AstFolders
        this.methodsArrayReport = []; // The array of methods reports
        this.template = undefined; // The HandleBar template used to generate the report
        this.astFolder = astFolder;
        this.astFolderService.astFolder = this.astFolder;
    }
    /**
     * Generates the folder's report
     */
    generateReport() {
        this.setMethodsArraySortedByDecreasingCognitiveCpx(this.astFolder);
        const reportTemplate = eol.auto(fs.readFileSync(`${options_model_1.Options.pathGeneseNodeJs}/json-ast-to-reports/templates/handlebars/folder-markdown-report.handlebars`, 'utf-8'));
        this.template = Handlebars.compile(reportTemplate);
        this.writeReport();
    }
    /**
     * Set the array of methods sorted by decreasing cognitive complexity
     * @param astFolder    // The AstFolder to analyse
     */
    setMethodsArraySortedByDecreasingCognitiveCpx(astFolder) {
        this.setTsFileReport(astFolder);
        this.setMethodsArrayReport(astFolder);
        this.methodsArrayReport = ast_method_service_1.AstMethodService.sortByDecreasingCognitiveCpx(this.methodsArrayReport);
    }
    /**
     * Recursion setting the array of methods reports of each subFolder
     * @param astFolder    // The AstFolder to analyse
     */
    setMethodsArrayReport(astFolder) {
        for (const subFolder of astFolder.children) {
            this.setTsFileReport(subFolder);
            this.setMethodsArrayReport(subFolder);
        }
    }
    /**
     * Recursion setting the array of methods reports of each subFolder's files
     * @param astFolder    // The AstFolder to analyse
     */
    setTsFileReport(astFolder) {
        for (const tsFile of astFolder.astFiles) {
            this.setAstMethodReport(tsFile);
        }
    }
    /**
     * Recursion setting the array of methods reports of each file's methods
     * @param astFile    // The AstFile to analyse
     */
    setAstMethodReport(astFile) {
        for (const astMethod of astFile.astMethods) {
            this.methodsArrayReport.push({
                cognitiveColor: astMethod.cognitiveStatus.toLowerCase(),
                cpxIndex: astMethod.cpxIndex,
                cyclomaticColor: astMethod.cyclomaticStatus.toLowerCase(),
                cyclomaticValue: astMethod.cyclomaticCpx,
                filename: astFile.name,
                linkFile: undefined,
                methodName: astMethod.name
            });
        }
    }
    /**
     * Fills the HandleBar's template
     */
    writeReport() {
        const template = this.template({
            rowFile: this.methodsArrayReport,
        });
        if (this.astFolder.relativePath) {
            file_service_1.createRelativeDir(this.astFolder.relativePath);
        }
        const pathOutDir = file_service_1.constructLink(options_model_1.Options.pathOutDir);
        const relativePath = file_service_1.constructLink(this.astFolder.relativePath);
        const pathReport = `${file_service_1.deleteLastSlash(pathOutDir)}/${file_service_1.deleteLastSlash(relativePath)}/folder-report.md`;
        try {
            fs.writeFileSync(pathReport, template, { encoding: "utf-8" });
        }
        catch (err) {
            console.log(err);
        }
    }
}
exports.AstFolderMarkdownReportService = AstFolderMarkdownReportService;
