"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InitGenerationService = void 0;
const fs = require("fs-extra");
const ast_file_generation_java_service_1 = require("./java/services/ast-file-generation-java.service");
const file_service_1 = require("../core/services/file.service");
const options_model_1 = require("../core/models/options.model");
const globals_const_1 = require("./globals.const");
const language_enum_1 = require("../core/enum/language.enum");
const ast_file_generation_service_1 = require("./ts/services/ast-file-generation.service");
/**
 * - AstFolders generation from Abstract Syntax Tree (AST) of its files (including files in subfolders)
 * - Conversion in JsonAst format
 */
class InitGenerationService {
    /**
     * Generates the AstFolder for a given folder
     * The tree is generated according to the Abstract Syntax TreeNode (AST) of the folder
     * @param  {string} path    // The path of the folder
     * @param  {Language} language
     * @returns JsonAstInterface
     */
    generateAll(path, language) {
        if (!path) {
            console.log('ERROR: no path.');
            return undefined;
        }
        return {
            astFolder: this.generateAstFolder(path, language)
        };
    }
    /**
     * Generates the AstFolder for the given source code
     * @param sourceCode
     * @returns {{astFolder: {path: string, astFiles: AstFileInterface[]}}}
     */
    generateAstFolderFromString(sourceCode) {
        sourceCode = `${sourceCode}\n`;
        const astFileGenerationService = new ast_file_generation_service_1.AstFileGenerationService();
        return {
            astFolder: {
                path: '',
                astFiles: [
                    astFileGenerationService.generateFromString(sourceCode)
                ]
            }
        };
    }
    /**
     * Generates the AstFolder corresponding to a given path and to its potential AstFolder parent
     * @param  {string} path              // The path of the AstFolder
     * @param  {Language} language
     * @returns AstFolderInterface
     */
    generateAstFolder(path, language) {
        let astFolder = {
            path: file_service_1.platformPath(path),
            astFiles: []
        };
        let initService;
        switch (language) {
            case language_enum_1.Language.JS:
            case language_enum_1.Language.TS:
            case language_enum_1.Language.JSX:
            case language_enum_1.Language.TSX:
                initService = new ast_file_generation_service_1.AstFileGenerationService();
                break;
            case language_enum_1.Language.JAVA:
                initService = new ast_file_generation_java_service_1.AstFileGenerationJavaService();
                break;
            default:
                initService = new ast_file_generation_service_1.AstFileGenerationService();
        }
        const filesOrDirs = fs.readdirSync(path);
        let currentFile = undefined;
        try {
            filesOrDirs.forEach((elementName) => {
                var _a;
                const pathElement = path + elementName;
                currentFile = pathElement;
                if (!options_model_1.Options.isIgnored(pathElement)) {
                    if (fs.statSync(pathElement).isDirectory() && !globals_const_1.LIMIT_GENERATIONS) {
                        astFolder.children = (_a = astFolder.children) !== null && _a !== void 0 ? _a : [];
                        astFolder.children.push(this.generateAstFolder(`${pathElement}/`, language));
                    }
                    else if (this.isFileToGenerate(pathElement, language)) {
                        astFolder.astFiles.push(initService.generate(pathElement, astFolder));
                    }
                }
            });
        }
        catch (e) {
            const [err, lines] = e.message.split('!!!');
            if (lines) {
                console.log(`Error in file: ${currentFile}\nAt line ${lines}`);
            }
            const error = new Error(err);
            error.stack = e.stack;
            throw error;
        }
        return astFolder;
    }
    /**
     * Returns true if a path corresponds to a file to generate in JsonAst
     * @param  {string} path  // The path of the file
     * @param  {Language} language
     * @returns boolean
     */
    isFileToGenerate(path, language) {
        return (file_service_1.getFileExtension(path) === language && !globals_const_1.LIMIT_GENERATIONS) || path === globals_const_1.DEV_MOCK;
    }
}
exports.InitGenerationService = InitGenerationService;
